% Estimating population aged 90 and over with the DA method. 
% This program has been produced to illustrate application of the DA method 
% with emphasis on clarity rather on efficiency. 
% 12-Jul-2004
% MATLAB Version 6.1
% ProjectID: DGMI 
% Reference
%   Andreev, Kirill F. A Method for Estimating Size of Population Aged 90 and over with 
%   Application to the 2000 U.S. Census Data. Demographic Research. 2004:107-121. ISSN: 0176-2516.

clc
close all
clear all

% global parameters
yearratios = 3;         % number of years to compute Das Gupta's death ratios
cyear      = 2000;      % current year, for this year population estimate (1 January) will be produced.
ageopen    = 90;        % start of open age group
cages      = [81 89];   % current range of ages; available population estimates in this age group will be used 
                        % for estimating beta-parameters 
agestart   = 80;        % starting age of the nrx sequence 

% load data, data set with the following columns: COHORT, AGE, POPULATION, DEATHS, TIMING, YEAR 
% United States, Females
fname = 'E:\Docs\SurvEst\data.csv';
data = [];
[data.cohort, data.age, data.population, data.deaths, data.timing, data.year] = textread(fname, '%f%f%f%f%f%f', ...
                'delimiter',    ',',                  ... 
                'emptyvalue',   NaN,                  ...
                'headerlines',  2                     ...  
                );
            
% compute death ratios, nrx            
ages = agestart : (max(data.age)-2);
D = []; % aggregated deaths
for a = ages
    dx = 0;     % deaths at age x, aggregated over calendar years
    dx1= 0;     % deaths at age x+1 corresponding to dx
    for y = 1:yearratios
        % get deaths
        e = data.age == a & data.year == (cyear-y-1) & data.timing == 2;
        i = find(e);
        dx = dx + data.deaths(i);
        %
        e = data.age == (a+1) & data.year == (cyear-y-1) & data.timing == 1;
        i = find(e);
        dx = dx + data.deaths(i);
        %
        e = data.age == (a+1) & data.year == (cyear-y) & data.timing == 2;
        i = find(e);
        dx1 = dx1 + data.deaths(i);
        %
        e = data.age == (a+2) & data.year == (cyear-y) & data.timing == 1;
        i = find(e);
        dx1 = dx1 + data.deaths(i);
    end
    D = [D; [dx dx1]];
end
ages = ages';
% nrx
D(find(D(:,1) == 0),:) = NaN;   % convert deaths equal zero to NaN's, just to avoid 'Division zero error'
nrx = D(:,2) ./ D(:,1);         
nrx(find(isnan(nrx))) = 0;      % convert nrx equal to NaN to zeros 

% estimate beta parameters
betas = sestdaest(data, nrx, ages, cyear, cages, agestart);

% compare estimated and available population in open age group
e = data.age >=ageopen & data.year == cyear & data.timing == 2;
i = find(e);
Nopenorg = sum(data.population(i));
%
N = sestdasurv(data, nrx, betas, ages, cyear, agestart);
e = N(:,1) >=ageopen;
i = find(e);
N = sum(N(i,2));

% command window ouptut 
disp(sprintf('The beta parameters'));
disp(sprintf('beta0 = %0.4f', betas(1)));
disp(sprintf('beta1 = %0.5f', betas(2)));
disp(sprintf('Population estimates at age %d and over', ageopen));
disp(sprintf('%16.2f - available population estimate', Nopenorg));
disp(sprintf('%16.2f - estimated by the DA method', N));
disp(sprintf('%15.2f%% - Relative error, %% to the DA method', (Nopenorg./N-1).*100));
